/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 2002 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `gpl.txt' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file README-BLUEWARE for more details. 
 * 
 */

#include <debug.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define OUTPUT_STREAM stdout

// These global variables need to be defined in a source file per library
//int trace_category = TRACECAT_ALL;
//int trace_level = LEVEL_HIGH;

static const char *
trace_category_name(TraceCategory category)
{
    // TraceCategory might have several bits set.
    if (category & TRACECAT_BT)
	    return "Bluetooth";
    else
	    return "";
}

void
vdebug_trace(TraceCategory category, TraceLevel level,
	     const char *file, int line,
	     const char *format, va_list val)
{
	if (!(trace_category & category) || !(trace_level & level))
		return;
	
	char new_format[1024];
	
	if(file) {
		int pos = strlen(file);
		while(--pos > 0) {
			if (*(file+pos) == '/') {
				file += pos + 1;
				break;
			}
		}
	}
	
	if (strlen(format) + (file ? strlen(file) : 0) > 1000)
		format = "<trace format too long>";
	
	char *new_format_ptr = new_format;
	const char *category_name = trace_category_name(category);
	if (*category_name)
		new_format_ptr += sprintf(new_format_ptr, "%s: ", category_name);
	
	if (file)
		new_format_ptr += sprintf(new_format_ptr, "%s:%d: ", file, line);
	
	strcpy(new_format_ptr, format);
	
	vfprintf(OUTPUT_STREAM, new_format, val);
}

void
debug_trace(TraceCategory category, TraceLevel level,
	    const char *file, int line,
	    const char *format, ...)
{
	va_list val;
	va_start(val, format);
	vdebug_trace(category, level, file, line, format, val);
	va_end(val);
}
