/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 2002 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `gpl.txt' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file README-BLUEWARE for more details. 
 * 
 */

#ifndef DEBUG_HH
#define DEBUG_HH

#include <assert.h>
#include <stdarg.h>


#define ASSERT assert

// Trace macro definitions.
//  #if defined(_DEBUG) || !defined(NDEBUG)
# define TRACECAT(category, level, format, args...) \
		debug_trace(category, level, __FILE__, __LINE__, format , ## args)
#define TRACE(level, format, args...) \
		debug_trace(TRACECAT_DEFAULT, level, NULL, -1, format , ## args)
//  #else
//  # define TRACECAT(category, level, format, args...)
//  # define TRACE(level, format, args...)
//  #endif


// This file combined with mazu/lib/debug.cc implements TRACE macros. //@GT
// The following global variables need to be defined in a source file per library.

// Global variables
extern unsigned trace_category; // bitwise-or of TraceCategory values
extern unsigned trace_level;    // bitwise-or of TraceLevel values

enum TraceCategory {
    TRACECAT_DEFAULT	= 0x00000001,
    TRACECAT_BT	        = 0x00000002,
    TRACECAT_ENERGY	= 0x00000004,
    TRACECAT_CHANNEL	= 0x00000008,
    TRACECAT_NONE	= 0x00000000,
    TRACECAT_ALL	= 0xFFFFFFFF
};

enum TraceLevel {
    LEVEL_PACKET	= 0x00000001, // send and receive of packets
    LEVEL_FUNCTION	= 0x00000002, // begin and end of functions
    LEVEL_MEMORY	= 0x00000004, // memory related
    LEVEL_ERROR		= 0x00000008, // errors
    LEVEL_LOW		= 0x00000010, // low priority
    LEVEL_MED		= 0x00000020, // medium priority 
    LEVEL_HIGH		= 0x00000040, // high prirority
    LEVEL_ACCT          = 0X00000080, // accounting info (for analysis)

    LEVEL_SPEC1         = 0x00010000,
    LEVEL_SPEC2         = 0x00020000,
    LEVEL_SPEC3         = 0x00040000,
    LEVEL_SPEC4         = 0x00080000,
    LEVEL_SPEC5         = 0x00100000,
    LEVEL_SPEC6         = 0x00200000,
    LEVEL_SPEC7         = 0x00400000,
    LEVEL_SPEC8         = 0x00800000,
    LEVEL_SPEC9         = 0x01000000,
    LEVEL_SPEC10        = 0x02000000,
    LEVEL_SPEC11        = 0x04000000,
    LEVEL_SPEC12        = 0x08000000,
    LEVEL_SPEC13        = 0x10000000,
    LEVEL_SPEC14        = 0x20000000,
    LEVEL_SPEC15        = 0x40000000,
    LEVEL_SPEC16        = 0x80000000,

    LEVEL_NONE		= 0x00000000,
    LEVEL_ALL		= 0xFFFFFFFF
};

void vdebug_trace(TraceCategory category, TraceLevel level, const char *file, int line, const char *format, va_list); 
void debug_trace(TraceCategory category, TraceLevel level, const char *file, int line, const char *format, ...);

#endif // DEBUG_HH
