#
#IBM Public License Version 1.0 
#
#THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS IBM PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT. 
#
#1. DEFINITIONS 
#
#"Contribution" means: 
#
#       a) in the case of International Business Machines Corporation ("IBM"), the Original Program, and 
#
#       b) in the case of each Contributor, 
#
#       i) changes to the Program, and
#
#       ii) additions to the Program;
#
#       where such changes and/or additions to the Program originate from and are distributed by that particular Contributor. A Contribution 'originates' from a Contributor if it
#       was added to the Program by such Contributor itself or anyone acting on such Contributor's behalf. Contributions do not include additions to the Program which: (i) are
#       separate modules of software distributed in conjunction with the Program under their own license agreement, and (ii) are not derivative works of the Program.
#
#  "Contributor" means IBM and any other entity that distributes the Program. 
#
#  "Licensed Patents " mean patent claims licensable by a Contributor which are necessarily infringed by the use or sale of its Contribution alone or when combined with the
#  Program. 
#
#  "Original Program" means the original version of the software accompanying this Agreement as released by IBM, including source code, object code and documentation, if
#  any. 
#
#  "Program" means the Original Program and Contributions. 
#
#  "Recipient" means anyone who receives the Program under this Agreement, including all Contributors. 
#
#  2. GRANT OF RIGHTS 
#
#       a) Subject to the terms of this Agreement, each Contributor hereby grants Recipient a non-exclusive, worldwide, royalty-free copyright license to reproduce, prepare
#       derivative works of, publicly display, publicly perform, distribute and sublicense the Contribution of such Contributor, if any, and such derivative works, in source code
#       and object code form.
#
#       b) Subject to the terms of this Agreement, each Contributor hereby grants Recipient a non-exclusive, worldwide, royalty-free patent license under Licensed Patents to
#       make, use, sell, offer to sell, import and otherwise transfer the Contribution of such Contributor, if any, in source code and object code form. This patent license shall
#       apply to the combination of the Contribution and the Program if, at the time the Contribution is added by the Contributor, such addition of the Contribution causes
#       such combination to be covered by the Licensed Patents. The patent license shall not apply to any other combinations which include the Contribution. No hardware per
#       se is licensed hereunder. 
#
#       c) Recipient understands that although each Contributor grants the licenses to its Contributions set forth herein, no assurances are provided by any Contributor that
#       the Program does not infringe the patent or other intellectual property rights of any other entity. Each Contributor disclaims any liability to Recipient for claims brought
#       by any other entity based on infringement of intellectual property rights or otherwise. As a condition to exercising the rights and licenses granted hereunder, each
#       Recipient hereby assumes sole responsibility to secure any other intellectual property rights needed, if any. For example, if a third party patent license is required to
#       allow Recipient to distribute the Program, it is Recipient's responsibility to acquire that license before distributing the Program.
#
#       d) Each Contributor represents that to its knowledge it has sufficient copyright rights in its Contribution, if any, to grant the copyright license set forth in this
#       Agreement. 
#
#  3. REQUIREMENTS 
#
#  A Contributor may choose to distribute the Program in object code form under its own license agreement, provided that: 
#
#       a) it complies with the terms and conditions of this Agreement; and
#
#       b) its license agreement:
#
#       i) effectively disclaims on behalf of all Contributors all warranties and conditions, express and implied, including warranties or conditions of title and non-infringement,
#       and implied warranties or conditions of merchantability and fitness for a particular purpose; 
#
#       ii) effectively excludes on behalf of all Contributors all liability for damages, including direct, indirect, special, incidental and consequential damages, such as lost
#       profits; 
#
#       iii) states that any provisions which differ from this Agreement are offered by that Contributor alone and not by any other party; and
#
#       iv) states that source code for the Program is available from such Contributor, and informs licensees how to obtain it in a reasonable manner on or through a medium
#       customarily used for software exchange. 
#
#  When the Program is made available in source code form: 
#
#       a) it must be made available under this Agreement; and 
#
#       b) a copy of this Agreement must be included with each copy of the Program. 
#
#  Each Contributor must include the following in a conspicuous location in the Program: 
#
#       Copyright  2002 International Business Machines Corporation, 
#       Massachusetts Institute of Technology, and others. All Rights Reserved. 
#
#  In addition, each Contributor must identify itself as the originator of its Contribution, if any, in a manner that reasonably allows subsequent Recipients to identify the originator
#  of the Contribution. 
#
#  4. COMMERCIAL DISTRIBUTION 
#
#  Commercial distributors of software may accept certain responsibilities with respect to end users, business partners and the like. While this license is intended to facilitate the
#  commercial use of the Program, the Contributor who includes the Program in a commercial product offering should do so in a manner which does not create potential liability
#  for other Contributors. Therefore, if a Contributor includes the Program in a commercial product offering, such Contributor ("Commercial Contributor") hereby agrees to defend
#  and indemnify every other Contributor ("Indemnified Contributor") against any losses, damages and costs (collectively "Losses") arising from claims, lawsuits and other legal
#  actions brought by a third party against the Indemnified Contributor to the extent caused by the acts or omissions of such Commercial Contributor in connection with its
#  distribution of the Program in a commercial product offering. The obligations in this section do not apply to any claims or Losses relating to any actual or alleged intellectual
#  property infringement. In order to qualify, an Indemnified Contributor must: a) promptly notify the Commercial Contributor in writing of such claim, and b) allow the Commercial
#  Contributor to control, and cooperate with the Commercial Contributor in, the defense and any related settlement negotiations. The Indemnified Contributor may participate in
#  any such claim at its own expense. 
#
#  For example, a Contributor might include the Program in a commercial product offering, Product X. That Contributor is then a Commercial Contributor. If that Commercial
#  Contributor then makes performance claims, or offers warranties related to Product X, those performance claims and warranties are such Commercial Contributor's
#  responsibility alone. Under this section, the Commercial Contributor would have to defend claims against the other Contributors related to those performance claims and
#  warranties, and if a court requires any other Contributor to pay any damages as a result, the Commercial Contributor must pay those damages. 
#
#  5. NO WARRANTY 
#
#  EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
#  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is solely responsible for determining the appropriateness of using and distributing the
#  Program and assumes all risks associated with its exercise of rights under this Agreement, including but not limited to the risks and costs of program errors, compliance with
#  applicable laws, damage to or loss of data, programs or equipment, and unavailability or interruption of operations. 
#
#  6. DISCLAIMER OF LIABILITY 
#
#  EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT,
#  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#  OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGES. 
#
#  7. GENERAL 
#
#  If any provision of this Agreement is invalid or unenforceable under applicable law, it shall not affect the validity or enforceability of the remainder of the terms of this
#  Agreement, and without further action by the parties hereto, such provision shall be reformed to the minimum extent necessary to make such provision valid and enforceable. 
#
#  If Recipient institutes patent litigation against a Contributor with respect to a patent applicable to software (including a cross-claim or counterclaim in a lawsuit), then any
#  patent licenses granted by that Contributor to such Recipient under this Agreement shall terminate as of the date such litigation is filed. In addition, If Recipient institutes
#  patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Program itself (excluding combinations of the Program with other
#  software or hardware) infringes such Recipient's patent(s), then such Recipient's rights granted under Section 2(b) shall terminate as of the date such litigation is filed. 
#
#  All Recipient's rights under this Agreement shall terminate if it fails to comply with any of the material terms or conditions of this Agreement and does not cure such failure in a
#  reasonable period of time after becoming aware of such noncompliance. If all Recipient's rights under this Agreement terminate, Recipient agrees to cease use and
#  distribution of the Program as soon as reasonably practicable. However, Recipient's obligations under this Agreement and any licenses granted by Recipient relating to the
#  Program shall continue and survive. 
#
#  IBM may publish new versions (including revisions) of this Agreement from time to time. Each new version of the Agreement will be given a distinguishing version number. The
#  Program (including Contributions) may always be distributed subject to the version of the Agreement under which it was received. In addition, after a new version of the
#  Agreement is published, Contributor may elect to distribute the Program (including its Contributions) under the new version. No one other than IBM has the right to modify this
#  Agreement. Except as expressly stated in Sections 2(a) and 2(b) above, Recipient receives no rights or licenses to the intellectual property of any Contributor under this
#  Agreement, whether expressly, by implication, estoppel or otherwise. All rights in the Program not expressly granted under this Agreement are reserved. 
#
#  This Agreement is governed by the laws of the State of New York and the intellectual property laws of the United States of America. No party to this Agreement will bring a
#  legal action under this Agreement more than one year after the cause of action arose. Each party waives its rights to a jury trial in any resulting litigation. 

proc create-tsf {} {
    global Sim

    set topo [new TopoConstructor/TSF]
    if { [info exists Sim(TSFdco)] != 0 } {
	$topo set d_coeff_ $Sim(TSFdco)
    }
    if { [info exists Sim(TSFdco)] != 0 } {
	$topo set heal_ $Sim(TSFheal)
    }

    return $topo
}

proc create-psf {num_resp inqTO} {
    set topo [new TopoConstructor/PSF]
    $topo set numResponses_ $num_resp
    $topo set inqTimeout_ $inqTO

    return $topo
}

proc create-sched {} {
    global Sim
    set sched 0

    #check whether Sched memeber exists in the array 
    if { [info exists Sim(Sched)] != 0 } {
	if {$Sim(Sched) == "LCS" || $Sim(Sched) == "TSS"} {
	    set sched [new TaskScheduler/LCS]
	} else {
	    set sched [new TaskScheduler]
	}
    } else {
	set sched [new TaskScheduler]
    }
    return $sched
}

proc create-master {num_resp inqTO masterIndex_ dev_id} {
        global Sim 

        if {$Sim(TSF) != 0 || $Sim(Topo) == "TSF"} {
		set topo [create-tsf]
	} else {
		set topo [create-psf $num_resp $inqTO]
	}
	set sched [create-sched]

	set Sim(dev:$dev_id) [new BTNode $dev_id $dev_id 0 1 $masterIndex_ [lindex $Sim(xpos_) $dev_id] [lindex $Sim(ypos_) $dev_id] $topo $Sim(chan) $sched]
	puts "create-master: $dev_id $Sim(dev:$dev_id)"

	if {$Sim(Trace) == "ON"} {
		#$Sim(dev:$dev_id) color "red"
		#$Sim(dev:$dev_id) shape "circle"
		#$Sim(dev:$dev_id) dump-namconfig
	}
}

proc create-slave {index diac inqScanOffset} {
	global Sim 

        if {$Sim(TSF) != 0 || $Sim(Topo) == "TSF"} {
		set topo [create-tsf]
	} else {
	        puts "There is no default topo formation scheme!\n"
	        exit 1;
		set topo [create-psf 0 0]
	        $topo set isOffset_ $inqScanOffset
	}
	set sched [create-sched]

	set Sim(dev:$index) [new BTNode $index $index $diac 0 0 [lindex $Sim(xpos_) $index] [lindex $Sim(ypos_) $index] $topo $Sim(chan) $sched]
	puts "create-slave: $index $Sim(dev:$index)"

	if {$Sim(Trace) == "ON"} {
		#$Sim(dev:$index) color "red"
		#$Sim(dev:$index) shape "circle"
		#$Sim(dev:$index) dump-namconfig
	}
}


proc config-trace {} {
	global ns Sim
	#set nam_fp $Sim(namFP)
	$ns trace-all $Sim(trFP)
}


proc config-app {src dst rate size} {
	global Sim ns
	set transport [lindex $Sim(Transport) $src]
	set Sim(trans:$src) [new Agent/$transport]
	set app [lindex $Sim(Application) $src]
	set Sim(appl:$src) [new Application/$app]

        # Set rate and pkt size //@GT
        #set rate [lindex $Sim(Rate) $src]
        #set size [lindex $Sim(Size) $src]

        if {$rate > 0} {
                $Sim(appl:$src) set rate_ $rate
	}
	if {$size > 0} {
                $Sim(appl:$src) set packetSize_ $size
	}

	set temp [string range $transport 0 2]
	$Sim(trans:$src) set packetSize_ $size

	if {$temp == "TCP"} {
	        # set initial window size to effect ssthresh Agent/TCP set window_ 20
	        # setting window to 85 is optimum for 5 to 6 hops in 20-node scatternets
	        #
	        $Sim(trans:$src) set window_ 90
	        $Sim(trans:$src) set ssthresh_ 90

	        # use this for 20-node scatternets
	        #$Sim(trans:$src) set window_ 170
	        #$Sim(trans:$src) set ssthresh_ 85

	        #$Sim(trans:$src) set window_ 120
	        #$Sim(trans:$src) set overhead_ 0.03
	        #$Sim(trans:$src) set newreno_changes_ 1
		set Sim(sink:$dst) [new Agent/TCPSink]
	} else {
		set Sim(sink:$dst) [new Agent/Null]
	}
	$ns attach-agent $Sim(dev:$src) $Sim(trans:$src)
	$ns attach-agent $Sim(dev:$dst) $Sim(sink:$dst)

	$ns connect $Sim(trans:$src) $Sim(sink:$dst)
	$Sim(dev:$src) attach-app $Sim(appl:$src)
	$Sim(appl:$src) attach-agent $Sim(trans:$src)

	# schedule application to start sending data at START
	$ns at $Sim(AppStart) "$Sim(appl:$src) start"

	# set trace targets 
	if { $Sim(AgentTrace) == "ON"} {
	    set temp [string range $transport 0 2]
	    if {$temp == "TCP"} {
		$ns add-agent-trace $Sim(trans:[expr $i-1]) tcp$i
		$ns monitor-agent-trace $Sim(trans:[expr $i-1])
		$Sim(trans:[expr $i-1]) attach $Sim(trFP)
		$Sim(trans:[expr $i-1]) tracevar t_seqno_

		#$Sim(trans:[expr $i-1]) tracevar cwnd_
		#$Sim(trans:[expr $i-1]) tracevar ssthresh_
		#$Sim(trans:[expr $i-1]) tracevar nrexmit_
	    }
	}

	# for now turn on all src-dst pairs
	if { $Sim(Trace) == "ON" } {
	    set tracedeq [$ns create-trace Deque $Sim(trFP) $Sim(dev:$src) $Sim(dev:$dst)]
	    set traceenq [$ns create-trace Enque $Sim(trFP) $Sim(dev:$src) $Sim(dev:$dst)]	
	    set tracedrp [$ns create-trace Drop  $Sim(trFP) $Sim(dev:$src) $Sim(dev:$dst)]
	    $Sim(dev:$src) set-tracetargets $traceenq $tracedeq $tracedrp
	}
}




