/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 2002 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `gpl.txt' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file README-BLUEWARE for more details. 
 * 
 */

#ifndef __util_h__
#define __util_h__

#include <stdarg.h>

#define  ALLOCP() gAllocPkt(__LINE__, __FILE__)
#define  ALLOCP1(sz_) gAllocPkt(sz_, __LINE__, __FILE__)
#define  FREEP(p_) gFreePkt(p_, __LINE__, __FILE__)
#define  COPYP(p_) gCopyPkt(p_, __LINE__, __FILE__)
#define  LOOKUP_TCLOBJ(obj, arg) \
			if((obj = TclObject::lookup(arg)) == NULL) { \
				fprintf(stderr, "%s lookup failed\n", argv[1]); \
				return TCL_ERROR; \
			} 

const int BUF_LEN = 1024;

typedef unsigned char uchar;
typedef unsigned short ushort;


/*==============================================================================
  extern functions 
  ============================================================================*/

class Packet;

extern Packet*   gAllocPkt(int lineno, const char* strfile);
extern Packet*   gAllocPkt(int size, int lineno, const char* strfile);
extern Packet*   gCopyPkt(Packet* p, int lineno, const char* strfile);
extern void      gFreePkt(Packet* p, int lineno, const char* strfile);



/*==============================================================================
  template functions 
  ============================================================================*/

template<class T> 
T cast(char* ptr) 
{ 
  *((T*)(ptr));
}

// Copy varaible parameters of T type into DATA. 
template<class T>
void copyVarArgs(char* data, int nArg, ...) 
{
  va_list ap;
  va_start(ap, nArgs);
  for(int i = 0; i < nArgs; i++)
    *((T*)(data+(i*sizeof(T)))) = va_arg(ap, T);
  va_end(ap);
}

template <class T>
bool
isWithin(T a, T b, double pcnt) 
{
	assert(pcnt >= 0 && pcnt <= 1 && a >= 0 && b >= 0);
	long int scale = 10000;
	long int test = (long int)(max(a,b) * scale);
	assert(test >= 0); // testing overflow
	return (labs((long int)(scale*(a-b))) <= scale*pcnt*b);
}

template <class T>
T
modulo(T a, T b) 
{
	assert(b > 0);
	int res;
	if (b % 2 == 0) {
		res = a & (b - 1);
	}
	else {
		res = a % b;
		if (res < 0)
			res += b;
	}

	assert(res >= 0);
	return (T)res;
}

// a pseudo random number between L and U.
template <class T>
T 
randRange(T l, T u) 
{
	double r = (double)rand();
	double intv = u - l + 1;
	if(r > RAND_MAX)
		assert(0);
	return (l+(T)(intv *  (r / (RAND_MAX+1.0))));
}


#endif





